@@ -2,6 +2,7 @@
 #
 # SPDX-License-Identifier: MIT
 
+from collections.abc import Callable
 from datetime import datetime, timezone
 from importlib.metadata import version
 import json
@@ -12,21 +13,45 @@ import subprocess
 from urllib.parse import urlparse
 from uuid import UUID, uuid4
 from debian import deb822
+from io import TextIOWrapper
 
 from debsbom.apt.cache import ExtendedStates, Repository
+from debsbom.bomwriter.bomwriter import BomWriter
 from debsbom.dpkg.package import ChecksumAlgo
 from debsbom.util.compression import Compression
 from debsbom.generate import Debsbom, SBOMType
 from debsbom.sbom import BOM_Standard
 
 
+class DebsbomLegacyProxy(Debsbom):
+    def __init__(self, sbom_types: set[SBOMType] | list[SBOMType] = [SBOMType.SPDX], **kwargs):
+        super().__init__(**kwargs)
+        self._sbom_types = sbom_types
+
+    def generate(
+        self,
+        out: str,
+        progress_cb: Callable[[int, int, str], None] | None = None,
+        validate: bool = False,
+        pkgs_stream: TextIOWrapper | None = None,
+    ):
+        self.scan(pkgs_stream)
+        for t in self._sbom_types:
+            outfile = out
+            if not outfile.endswith(f".{t}.json"):
+                outfile += f".{t}.json"
+            bom = super().generate(t, progress_cb)
+            BomWriter.create(t).write_to_file(bom, Path(outfile), validate)
+
+
 @pytest.fixture
 def sbom_generator():
     def setup_sbom_generator(
         test_root: Path,
         uuid: UUID | None = None,
         timestamp: datetime | None = None,
-        sbom_types: list[SBOMType] = [SBOMType.SPDX, SBOMType.CycloneDX],
+        with_licenses: bool = False,
+        sbom_types: list[SBOMType] = list(SBOMType),
     ) -> Debsbom:
         url = urlparse("http://example.org")
         if uuid is None:
@@ -34,7 +59,7 @@ def sbom_generator():
         if timestamp is None:
             timestamp = datetime(1970, 1, 1, tzinfo=timezone.utc)
 
-        return Debsbom(
+        return DebsbomLegacyProxy(
             distro_name="pytest-distro",
             distro_arch="amd64",
             sbom_types=sbom_types,
@@ -42,12 +67,16 @@ def sbom_generator():
             spdx_namespace=url,
             cdx_serialnumber=uuid,
             timestamp=timestamp,
+            with_licenses=with_licenses,
         )
 
     return setup_sbom_generator
 
 
 def test_tree_generation(tmpdir, sbom_generator):
+    _spdx_tools = pytest.importorskip("spdx_tools")
+    _cyclonedx = pytest.importorskip("cyclonedx")
+
     timestamp = datetime(1970, 1, 1, tzinfo=timezone.utc)
     uuid = uuid4()
     dbom = sbom_generator("tests/root/tree", uuid, timestamp)
@@ -75,6 +104,9 @@ def test_tree_generation(tmpdir, sbom_generator):
 
 
 def test_dependency_generation(tmpdir, sbom_generator):
+    _spdx_tools = pytest.importorskip("spdx_tools")
+    _cyclonedx = pytest.importorskip("cyclonedx")
+
     dbom = sbom_generator("tests/root/dependency")
     outdir = Path(tmpdir)
     dbom.generate(str(outdir / "sbom"), validate=True)
@@ -138,19 +170,23 @@ def test_dependency_generation(tmpdir, sbom_generator):
 
 
 def test_standard_bom(tmpdir):
+    _cyclonedx = pytest.importorskip("cyclonedx")
+
     dbom = Debsbom(
         distro_name="pytest-distro",
         distro_arch="amd64",
-        sbom_types=[SBOMType.CycloneDX],
         root="tests/root/dependency",
         spdx_namespace=urlparse("http://example.org"),
         cdx_serialnumber=uuid4(),
         timestamp=datetime(1970, 1, 1, tzinfo=timezone.utc),
         cdx_standard=BOM_Standard.STANDARD_BOM,
     )
+    dbom.scan()
     outdir = Path(tmpdir)
-    dbom.generate(str(outdir / "sbom"), validate=True)
-    with open(outdir / "sbom.cdx.json") as file:
+    bom = dbom.generate(SBOMType.CycloneDX)
+    bomfile = outdir / "sbom.cdx.json"
+    BomWriter.create(SBOMType.CycloneDX).write_to_file(bom, bomfile, validate=True)
+    with open(bomfile) as file:
         cdx_json = json.loads(file.read())
         s_bom = cdx_json["definitions"]["standards"][0]
         assert s_bom["bom-ref"] == "standard-bom"
@@ -158,6 +194,9 @@ def test_standard_bom(tmpdir):
 
 
 def test_homepage_regression(tmpdir, sbom_generator):
+    _spdx_tools = pytest.importorskip("spdx_tools")
+    _cyclonedx = pytest.importorskip("cyclonedx")
+
     dbom = sbom_generator("tests/root/homepage-lowercase")
     outdir = Path(tmpdir)
     dbom.generate(str(outdir / "sbom"), validate=True)
@@ -172,6 +211,9 @@ def test_homepage_regression(tmpdir, sbom_generator):
 
 
 def test_apt_source_pkg(tmpdir, sbom_generator):
+    _spdx_tools = pytest.importorskip("spdx_tools")
+    _cyclonedx = pytest.importorskip("cyclonedx")
+
     dbom = sbom_generator("tests/root/apt-sources")
     outdir = Path(tmpdir)
     dbom.generate(str(outdir / "sbom"), validate=True)
@@ -222,6 +264,9 @@ def test_apt_source_pkg(tmpdir, sbom_generator):
 
 
 def test_apt_pkgs_stream(tmpdir, sbom_generator):
+    _spdx_tools = pytest.importorskip("spdx_tools")
+    _cyclonedx = pytest.importorskip("cyclonedx")
+
     dbom = sbom_generator("tests/root/apt-sources")
     with open("tests/data/pkgs-stream", "r") as stream:
         outdir = Path(tmpdir)
@@ -250,10 +295,12 @@ def test_apt_extended_states():
     assert noes.is_manual("foo", "amd64")
 
 
-def test_apt_cache_parsing():
+@pytest.mark.parametrize("origin", ["Debian", "Local"])
+def test_apt_cache_parsing(origin):
     apt_lists_dir = "tests/root/apt-sources/var/lib/apt/lists"
-    repo = next(Repository.from_apt_cache(apt_lists_dir))
-    src_pkgs = list(repo.sources(lambda p: p.name == "binutils"))
+    repos = list(Repository.from_apt_cache(apt_lists_dir))
+    deb_repo = next(filter(lambda r: r.origin == origin, repos))
+    src_pkgs = list(deb_repo.sources(lambda p: p.name == "binutils"))
     assert len(src_pkgs) == 1
     # this data is only available in apt sources deb822 data
     assert "binutils-for-host" in src_pkgs[0].binaries
@@ -268,6 +315,9 @@ compressions = ["bzip2", "gzip", "xz", "zstd", "lz4"]
 
 @pytest.mark.parametrize("tool", compressions)
 def test_apt_lists_compression(tmpdir, sbom_generator, tool):
+    _spdx_tools = pytest.importorskip("spdx_tools")
+    _cyclonedx = pytest.importorskip("cyclonedx")
+
     comp = Compression.from_tool(tool)
 
     # create a temporary rootfs copied from tests/root/apt-sources and
@@ -321,9 +371,16 @@ def test_apt_lists_compression(tmpdir, sbom_generator, tool):
                     "algorithm": "MD5",
                     "checksumValue": "041580298095f940c2c9c130e0d6e149",
                 } in pkg["checksums"]
+                assert {
+                    "algorithm": "SHA512",
+                    "checksumValue": "98432f5466d899b96031702e3bb3d75b398a691b295bc248edbc5254f5460f1f4affce98246e95cd51f10e09636e7cdee0b6df2423842179314236485b3fb1d1",
+                } in pkg["checksums"]
 
 
 def test_pkglist_apt_cache(tmpdir, sbom_generator):
+    _spdx_tools = pytest.importorskip("spdx_tools")
+    _cyclonedx = pytest.importorskip("cyclonedx")
+
     dbom = sbom_generator("tests/root/apt-sources")
     with open("tests/data/pkgs-custom-stream", "r") as stream:
         outdir = Path(tmpdir)
@@ -338,6 +395,9 @@ def test_pkglist_apt_cache(tmpdir, sbom_generator):
 
 
 def test_residual_config_packages(tmpdir, sbom_generator):
+    _spdx_tools = pytest.importorskip("spdx_tools")
+    _cyclonedx = pytest.importorskip("cyclonedx")
+
     dbom = sbom_generator("tests/root/apt-sources")
     outdir = Path(tmpdir)
     dbom.generate(str(outdir / "sbom"), validate=True)
@@ -364,3 +424,77 @@ def test_illformed_sources():
     # parse incomplete packages. Must not raise
     assert len(list(Repository._make_srcpkgs([deb822.Packages()]))) == 0
     assert len(list(Repository._make_binpkgs([deb822.Packages()]))) == 0
+
+
+def test_license_information(tmpdir, sbom_generator):
+    _spdx_tools = pytest.importorskip("spdx_tools")
+    _cyclonedx = pytest.importorskip("cyclonedx")
+
+    dbom = sbom_generator("tests/root/copyright", with_licenses=True)
+    outdir = Path(tmpdir)
+    dbom.generate(str(outdir / "sbom"), validate=True)
+    with open(outdir / "sbom.spdx.json") as file:
+        spdx_json = json.loads(file.read())
+        apt_pkg = None
+        packages = spdx_json["packages"]
+        for package in packages:
+            for ref in package.get("externalRefs") or []:
+                if (
+                    ref["referenceCategory"] == "PACKAGE_MANAGER"
+                    and "arch=source" in ref["referenceLocator"]
+                ):
+                    apt_pkg = package
+                    break
+            if apt_pkg:
+                break
+        assert apt_pkg
+        assert (
+            apt_pkg["licenseDeclared"]
+            == "BSD-3-Clause AND GPL-2.0-only AND GPL-2.0-or-later AND MIT"
+        )
+    with open(outdir / "sbom.cdx.json") as file:
+        spdx_json = json.loads(file.read())
+        apt_component = list(filter(lambda c: "arch=source" in c["purl"], spdx_json["components"]))[
+            0
+        ]
+        lic = apt_component["licenses"][0]
+        assert lic["acknowledgement"] == "declared"
+        assert lic["expression"] == "BSD-3-Clause AND GPL-2.0-only AND GPL-2.0-or-later AND MIT"
+
+
+def test_virtual_package(tmpdir, sbom_generator):
+    _spdx_tools = pytest.importorskip("spdx_tools")
+    _cyclonedx = pytest.importorskip("cyclonedx")
+
+    dbom = sbom_generator("tests/root/virtual-packages", with_licenses=True)
+    outdir = Path(tmpdir)
+    dbom.generate(str(outdir / "sbom"), validate=True)
+    with open(outdir / "sbom.spdx.json") as file:
+        spdx_json = json.loads(file.read())
+        relationships = spdx_json["relationships"]
+
+        found = False
+        for relationship in relationships:
+            if (
+                relationship["spdxElementId"] == "SPDXRef-foo-amd64"
+                and relationship["relationshipType"] == "DEPENDS_ON"
+            ):
+                assert relationship["relatedSpdxElement"] == "SPDXRef-bar-plus-amd64"
+                found = True
+                break
+        assert found
+
+    with open(outdir / "sbom.cdx.json") as file:
+        cdx_json = json.loads(file.read())
+
+        found = False
+        for dependency in cdx_json["dependencies"]:
+            if dependency["ref"] == "pkg:deb/debian/foo@1.0?arch=amd64":
+                for dep in dependency["dependsOn"]:
+                    assert (
+                        dep == "pkg:deb/debian/bar-plus@1.0?arch=amd64"
+                        or dep == "pkg:deb/debian/foo@1.0?arch=source"
+                    )
+                found = True
+                break
+        assert found
